/*
##################################################################################################
#
#   Travelling Salesman Problem with Kohonen network
#   Sommersemester 1996 
#   Neuronale Netze & ihre Anwendungen
#   Michael Ratz, Johannes Beck 
#   Java - Portierung: Johannes Beck
##################################################################################################
*/

import java.lang.Math;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Color;

/* Sample of a city */
class sample {
        public float x,y;
        sample next; 
        };

/* a node in the neural network */
class node {
        public float x,y;
        public node right;
        public node left;     
        public int life;
        public int inhibitation;
        public int isWinner;

        /* the distance of the euklidian points */
        float potential(sample s)
                {
                return (s.x - x) * (s.x - x) + (s.y - y) * (s.y - y);
                }

        /* moves a single node in direction to the sample */
        synchronized void move(sample city, float value)
                {
                x += value *(city.x - x );
                y += value *(city.y - y );
                }
        /* computes the number of nodes between the to nodes on the ring */
        int distance(node j,int length)
        {
        int right = 0;
        int left  = 0;
        node actual = j;

        while (actual!=this)
                {
                actual=actual.left;
                left++;
                }
        right = length - left;
        return ((left<right) ? left : right);                
        };

        /* a line between to nodes              */
        synchronized void drawLine(node p2,Graphics g,Dimension size) 
                {
                 int offX   = size.width  / 10;     
                 int offY   = size.height / 10;
                 int lx     = size.width  * 4 / 5;
                 int ly     = size.height * 4 / 5;
                
                 g.drawLine((int)(lx*x    + offX),(int)(ly*y    + offY),
                            (int)(lx*p2.x + offX),(int)(ly*p2.y + offY) );
                };
};      

/* the neural network as a ring of neurons */
class ring {
        public node start;
        public int length;

        /* moves all nodes to in direction of the sample */  
        synchronized void moveAllNodes(sample city, float G)
        {
        node j   =start;
        node best=findMinimum(city);

        for(int i=0; i<length; i++)                     
                {
//                if (j.inhibitation>0)
                        j.move(city,f(G,best.distance(j,length)));            
                j=j.right;
                }
        };

        /* finds the node with the least distance to the sample */
        node findMinimum(sample x)
        {
         float actual;

         node test=start;
         node best=test;
         float min = test.potential(x);
         for (int i=1; i<length; i++)
                {
                test=test.right;       
                actual = test.potential(x);
                if (actual<min) {
                                min=actual;             
                                best=test;
                                }
                }
         best.isWinner++;
         return best;
        };

        /* deletes a node */
        synchronized void deleteNode(node test)
        {
         node last=test.left;
         node next=test.right;

         if (last!=null)
                last.right = next;
         if (next!=null)
                next.left  = last;        
         if (next==test)
                next=null;
         if (start==test)
                start=next;
         length--;               
        };

        /* a node is duplicated & inserted into the ring */
        synchronized void duplicateNode(node test)
        {
         node neu,next;

         next = test.left;
         neu = new node();
         if (neu==null) return;
         neu.left  = next;
         neu.right = test;
         test.left = neu;
         next.right= neu;       
         neu.x     = test.x;
         neu.y     = test.y;
         neu.life  = 3;
         neu.inhibitation = test.inhibitation = 1;
         neu.isWinner = 0;
         length++;       
        };

        /* length of tour       */
        synchronized float ring_length()
        {
         float dist=0.0f;
         node x=start;
         node c=x.left;

         for (int i=0; i<length; i++)
                {
                dist += (float)Math.sqrt((double)((x.x-c.x) * (x.x-c.x) + (x.y-c.y) * (x.y-c.y)));
                x=c;
                c=c.left;
                }
         return dist;
        };

        /* all nodes connected with lines       */
        synchronized void draw(Graphics g,Dimension size)
        {
         node actual = start;
         node next   = actual.left;

         for (int i=0; i<length; i++)
                {
                actual.drawLine(next,g,size);
                actual =next;
                next   =next.left;
                }
        };

        float f(float G,int n)
        {        
         return (float)( 0.70710678 * Math.exp( -(n*n) / (G*G)) );
        };
};

/* the simulator containing all the data for the simulation */
public class Tsp extends java.applet.Applet implements Runnable {
 public int      N;              /* Number of cities.                    */
 public int      cycle;          /* Number of complete survey done       */
        int      maxCycles;      /* Number of complete suerveys          */
 public sample   city;           /* the samples                          */
 public ring     neurons;        /* the neurons                          */
 public float    alpha;          /* learning rate                        */
 public float    G;              /* gain                                 */
        int      update;         /* screen update                        */
        float    lastLength;     /* length of tour                       */
        Image    offscreenImg;
        Graphics offscreenG;

        Thread runner;

        /* creates the first node (ring) */
        void createFirstNeuron()
        {
         node neu = new node();

         neurons = new ring();
         if (neu==null) return;
         neu.right      =neu;
         neu.left       =neu;
         neu.x          =0.5f;
         neu.y          =0.5f;
         neu.life       =3;
         neu.inhibitation=0;
         neu.isWinner   =0;
         neurons.length =1;
         neurons.start  =neu;
        };

        /* deletes all nodes */
        void deleteAllNeurons()
        {
         while (neurons.start!=null)
                neurons.deleteNode(neurons.start);
        };

        /* deletes n cities starting with the given */ 
        void deleteAllCities()
        {
         sample actual = city;
         sample last   = null;
         for(int i=0; i<N; i++)
                {
                last   = actual;
                actual = actual.next;
                }        
        };

        /* prints positions of cities & nodes */
        void print()
        {
         node n;
        
         for (int i=0; i<N; i++)
                {
                System.out.println("City: "+i+" ("+city.x+","+city.y+")");
                city=city.next;
                }
         n=neurons.start;
         for (int i=0; i<neurons.length; i++)
                {
                System.out.println("Node: "+i+"("+n.x+","+n.y+")");
                n=n.right;
                }        
        };
        
        /* creates & displaces randomly a given number of cities, returns the first */ 
        void createRandomCities()
        {
         sample last;
         sample neu  = null;

         city=null;
         for (int i=0; i<N; i++)
                {
                last = neu;
                neu = new sample();
                if (neu==null) return;
                neu.next = null;
                neu.x    = (float)Math.random();
                neu.y    = (float)Math.random();
                if (city==null)
                        city=neu;
                if (last!=null)
                        last.next=neu;
                }
         if (neu!=null)
                neu.next = city;
        };

        /* after moving creating & deleting is done */
        void surveyFinish()
        {
         node test = neurons.start;
         for (int i=0; i < neurons.length; i++)
                {
                test.inhibitation=0;
                switch (test.isWinner)
                        {
                        case 0: test.life--;
                                if (test.life==0) 
                                        neurons.deleteNode(test);
                                break;
                        case 1: test.life=3;   
                                break;
                        default:
                                test.life=3;
                                neurons.duplicateNode(test);
                                break;
                        }               
                test.isWinner=0;
                test=test.right;
                }               
        };

        /* 1 cycle in the simulation */ 
        int surveyRun()
        {
         float length;
         int    done = 0;
         sample c = city;
         
         for (int i=0; i<N; i++)
                {
                neurons.moveAllNodes(c,G);
                c=c.next;
                }
         surveyFinish();
         G = G * (1 - alpha);
         if (cycle++ % update == 0)
                {
                length = neurons.ring_length();
                repaint();
                if (length==lastLength)
                        done=1;
                else lastLength=length;
                }       
        return done;
        };

        /* prints positions of cities & nodes   */
        void printStatus(Graphics g)
        {
         int offX   = size().width  / 10;     
         int offY   = size().height / 10;
         int ly     = size().height * 4 / 5;
        
         g.drawString("cycle = "+cycle+" of "+maxCycles+"   cities = "+N+" nodes = "+neurons.length,offX,offY+ly+20);
         g.drawString("alpha = "+alpha+" G = "+G+"    length of tour = "+lastLength,offX,offY+35+ly);
        };

        /* all cities as small circles          */
        void drawCities(Graphics g)
        {
         int offX   = size().width  / 10;     
         int offY   = size().height / 10;
         int lx     = size().width  * 4 / 5;
         int ly     = size().height * 4 / 5;

         sample c=city;
         g.drawString("Traveling Salesman Problem with Kohonen network",offX,10);
         for (int i=0; i<N; i++)  
                {
                g.drawOval( (int)(lx*c.x + offX),(int)(ly*c.y + offY),3,3);
                c=c.next;      
                }        
        };

        public void init()
        {      
         String par;
        
         par = getParameter("CITIES");
         if (par!=null) 
              N=Integer.parseInt(par); 
         else N=10; 
         par = getParameter("MAXCYCLES");
         if (par!=null) 
              maxCycles=Integer.parseInt(par); 
         else maxCycles=1000; 
         par = getParameter("ALPHA");
         if (par!=null) 
              alpha=Float.valueOf(par).floatValue(); 
         else alpha=0.05f; 
         par = getParameter("UPDATE");
         if (par!=null) 
              update=Integer.parseInt(par); 
         else update=5; 
         par = getParameter("GAIN");
         if (par!=null) 
              G=Float.valueOf(par).floatValue();
         else G=50f; 
         cycle      = 0;
         lastLength = -1;
         createFirstNeuron();
         createRandomCities();
         offscreenImg = createImage(size().width,size().height);
         offscreenG   = offscreenImg.getGraphics();
        };

        public void start()
        {
         if (runner==null)
                {
                runner = new Thread(this);
                runner.start();
                }
        };
        
        public void run()
        {
         while ( (cycle<maxCycles) && (surveyRun()==0))
                {
                try  { Thread.sleep(10); } catch(InterruptedException e) {}
                }
         repaint();
        };

        public void stop()
        {
         if (runner!=null)
                {
                runner.stop();
                runner = null;        
                }
//         print();
         deleteAllCities();
         deleteAllNeurons();
        };
        
        public void paint(Graphics g) 
        {
         offscreenG.setColor(getBackground());
         offscreenG.fillRect(0,0,size().width,size().height);
         offscreenG.setColor(getForeground());
         drawCities(offscreenG);
         neurons.draw(offscreenG,size());
         printStatus(offscreenG);
         g.drawImage(offscreenImg,0,0,this);
        }
        
        public void update(Graphics g)
        {
         paint(g);
        }      
};



