/*
##################################################################################################
#
#   Travelling Salesman Problem with Kohonen network
#   Sommersemester 1996 
#   Neuronale Netze & ihre Anwendungen
#   Michael Ratz, Johannes Beck 
#
##################################################################################################
*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include "tsp.h"
#include "tsp_parser.h"

static graphic g(1);

float f(float G,int n)
	{
	return ( 0.70710678 * expf( -(n*n) / (G*G)) ); 
	}

int node::distance(node *j,int length)
	{
	int right = 0;
	int left  = 0;
	node *actual = this;

	while (actual!=j)
		{
		actual=actual->left;
		left++;
		}
	right = length - left;
	return ((left<right) ? left : right);
	}

void ring::move_all_nodes(sample *city, float G)
	{
	int i=0;
	node *j   =start;
	node *best=find_minimum(city);

	while (i < length )			
		{
		if (!(j->inhibitation) )
			j->move(city,f(G,best->distance(j,length)));		
		j=j->right;
		i++;
		}
	}
	
float ring::ring_length()
        {
	float dist=0.0;
	node *x=start;
	node *c=x->left;

	for (int i=0; i<length; i++)
		{
		dist = dist + sqrt( (x->x - c->x) * (x->x - c->x) + (x->y - c->y) * (x->y - c->y));
        	x=c;
		c=c->left;
		}
	return dist;
	}

node* ring::find_minimum(sample *x)
	{
	float actual;

	node *test=start;
	node *best=test;
	float min = test->potential(x);
	for (int i=1; i<length; i++)
		{
		test=test->right;	
		actual = test->potential(x);
		if (actual<min) {
				min=actual;		
				best=test;
				}
		}
	best->is_winner++;
	return best;
	}

int node::draw_line(node *p2,graphic &g)
	{
	return (line_id=g.draw_line(x,y,p2->x,p2->y));
	}

void ring::delete_node(node *test)
	{
	node *last=test->left;
	node *next=test->right;

	g.deleteLine[g.deleteCounter++]=test->line_id;		  
	if (last)
		last->right = next;
	if (next)
		next->left  = last;	   
	if (next==test)
		next=NULL;
	if (start==test)
		start=next;
	delete test;
	length--;		
	}

void ring::duplicate_node(node *test)
	{
	node *neu,*next;

	next = test->right;
	neu = new node();
	if (!neu) return;
	neu->right = next;
	neu->left  = test;
	test->right= neu;
	next->left = neu;	
	neu->x	   = test->x + (next->x-test->x) / 100.0 ;
	neu->y	   = test->y + (next->y-test->y) / 100.0 ;
	neu->life  = 3;
	neu->inhibitation = test->inhibitation = 1;
	neu->is_winner = 0;
	neu->line_id=0;
	length++;	
	}

void ring::draw(graphic &g)
	{
	node *actual = start;
	node *next   = actual->left;

	for (int i=0; i<length; i++)
		{
		actual->draw_line(next,g);
		actual =next;
		next   =next->left;
		}
	}

void ring::undraw(graphic &g)
	{
	node *actual = start;
        
	for (int i=0;i<length;i++)
                {
		g.delete_line(actual->line_id);
		actual->line_id=0;		  
                actual =actual->left;
                }
	if (g.deleteCounter)
	 for (int i=0; i<g.deleteCounter; i++)
		g.delete_line(g.deleteLine[i]);
	g.deleteCounter=0;
        }

void simulator::survey_finish()
	{
	node *test;
	int i=0;

	test=neurons->start;
	while (i < neurons->length)
		{
		test->inhibitation=0;
		switch (test->is_winner)
			{
			case 0 :
				test->life--;
				if (test->life==0) 
					neurons->delete_node(test);
				break;
			case 1 : 
				test->life=3;	
				break;
			default :
				test->life=3;
				neurons->duplicate_node(test);
				break;
			}		
		test->is_winner=0;
		i++;
		test=test->right;
		}		
	}		

int simulator::survey_run()
{
	float length;
	int    done;
	sample *c = city;
	
	done = 0;
	for (int i=0; i<N; i++)
		{
		neurons->move_all_nodes(c,G);
		c=c->next;
		}
	survey_finish();
	G = G * (1 - alpha);
	if (cycle % update == 0)
		{
		neurons->undraw(g);
		neurons->draw(g);
		length = print_status(g);
		if (length==last_length)
			done=1;
		else last_length=length;
		}	
	cycle++;
	return done;
}

void simulator::createFirstNeuron()
{
	node *neu = new node();

	neurons = new ring();
	if (!neu) 
		return;
	neu->right =neu;
	neu->left  =neu;
	neu->x	   =0.5;
	neu->y	   =0.5;
	neu->life  =3;
	neu->inhibitation=0;
	neu->is_winner=0;
	neurons->length=1;
	neurons->start=neu;
}

void simulator::deleteAllNeurons()
{
	while (neurons->start)
		neurons->delete_node(neurons->start);
	delete neurons;
}

void simulator::deleteAllCities()
{
	sample *actual = city;
	sample *last   = NULL;
	for(int i=0; i<N; i++)
		{
		last   = actual;
		actual = actual->next;
		delete last;
		}
}

void simulator::createRandomCities()
{
	sample *last;
	sample *neu  = NULL;

	city=NULL;
	srandom((unsigned)time(NULL));	
	for (int i=0; i<N; i++)
		{
		last = neu;
		neu = new sample();
		if (!neu) return;
		neu->next = NULL;
		neu->x    = (float)(random() % 100) / 100.0;
		neu->y    = (float)(random() % 100) / 100.0;
		if (!city)
			city=neu;
		if (last)
			last->next=neu;
		}
}

void simulator::draw_cities(graphic &g)
	{
    	char buf[80];
    
    	sprintf(buf, "Traveling Salesman Problem with Kohonen network");
	
    	g.statusT[2] = g.p->CreateText(buf, 50, 15 );
	sample *c=city;
	for (int i=0; i<N; i++)  
       		{
		g.p->CreateStaticCircle( int(g.lambda*c->x + g.offset),int(g.lambda*c->y +g.offset), 3);
   		c=c->next;	
		}
	}

void simulator::print()
{
	int i=0;
	node *n;
	
	for (i=0; i<N; i++)
		{
		printf("City: %d (%f,%f)\n",i,city->x,city->y);
		city=city->next;
		}
	n=neurons->start;
	for (i=0; i<neurons->length; i++)
		{
		printf("Node: %d (%f,%f)\n",i,n->x,n->y);
		n=n->right;
		}	
}

/* prints positions of cities & nodes */
float simulator::print_status(graphic &g)
{
    char buf[80];
    float length =  neurons->ring_length();
    
    sprintf(buf, "cycle = %5d of %5d    cities = %4d nodes = %4d",
	cycle,maxCycles, N , neurons->length );
    if (g.statusT[0]) 
	g.p->KillObj(g.statusT[0]);
    g.statusT[0] = g.p->CreateText(buf, 30, g.lambda + g.offset + 20 );

    sprintf(buf, "alpha = %3f G = %3f    length of tour = %4f",
	alpha , G, length  );
    if (g.statusT[1]) 
	g.p->KillObj(g.statusT[1]);
    g.statusT[1] = g.p->CreateText(buf, 30, g.lambda + g.offset + 35 );
    return length;
}

/* main program, initializes and starts the simulation */
int main(int argc,char **argv)
{
	simulator *sim;
	
	if (argc==2)
		sim = readMap(argv[1]);
	else	{
		fprintf(stderr,"Usage: tsp mapfile\n");
		exit(-1);
		} 
	

	g.graphic(sim->screensize);
	sim->draw_cities(g);
	sim->last_length= -1;
	while (sim->cycle < sim->maxCycles && !sim->survey_run());
	
	sim->neurons->undraw(g);
	sim->neurons->draw(g);
	sim->print_status(g);
	getchar();

	/* last but not least	 	*/
	sim->deleteAllCities();
	sim->deleteAllNeurons();
	delete sim;
	
	g.~graphic();
	return 0;	
}
