/*
##################################################################################################
#
#   Travelling Salesman Problem with Kohonen network
#   Sommersemester 1996 
#   Neuronale Netze & ihre Anwendungen
#   Michael Ratz, Johannes Beck 
#
##################################################################################################
*/

#ifndef TSP_H
#define TSP_H
#include "graph.h"

/* Classes */

/* Sample of a city */
class sample {
 public:
	float x,y;
	sample *next; 
	};

/* a node in the neural network */
class node {
 public:
	float x,y;
	node *right;
	node *left;	
	int line_id;
	int life;
	int inhibitation;
	int is_winner;

	/* the distance of the euklidian points */
	float potential(sample *s)
		{
		return (s->x - x) * (s->x - x) + (s->y - y) * (s->y - y);
		}

	/* moves a single node in direction to the sample */
	void move(sample *city, float value)
		{
		x = x + value *(city->x - x);
		y = y + value *(city->y - y);
		}
	/* computes the number of nodes between the to nodes on the ring */
	int distance(node *,int);

	/* a line between to nodes		*/
	int draw_line(node *,graphic &g);
};	

/* the neural network as a ring of neurons */
class ring {
 public:
	node *start;
	int length;

	/* moves all nodes to in direction of the sample */  
	void move_all_nodes(sample *, float);

	/* finds the node with the least distance to the sample */
	node* find_minimum(sample *);

	/* deletes a node */
	void delete_node(node *);

	/* a node is duplicated & inserted into the ring */
	void duplicate_node(node *);

	/* length of tour	*/
	float ring_length();

	/* all nodes connected with lines	*/
	void draw(graphic &g);

	/* deletes all lines			*/
	void undraw(graphic &g);
	};

/* the simulator containing all the data for the simulation */
class simulator {
 public:
	int N; 		/* Number of cities.			*/
	int cycle;	/* Number of complete survey done	*/
	int maxCycles;	/* Number of complete suerveys 		*/
	sample *city;	/* the samples				*/
	ring *neurons;	/* the neurons				*/
	float alpha;	/* learning rate			*/
	float G;	/* gain					*/
	int update;	/* screen update			*/
	int screensize; /* size of the window			*/
	float last_length; /* length of tour 			*/

	/* creates the first node (ring) */
	void createFirstNeuron();

	/* deletes all nodes */
	void deleteAllNeurons();

	/* deletes n cities starting with the given */ 
	void deleteAllCities();

	/* prints positions of cities & nodes */
	void print();
	
	/* creates & displaces randomly a given number of cities, returns the first */ 
	void createRandomCities();

	/* after moving creating & deleting is done */
	void survey_finish();

	/* 1 cycle in the simulation */ 
	int survey_run();

	/* prints positions of cities & nodes 	*/
	float print_status(graphic &g);

	/* all cities as small circles		*/
	void draw_cities(graphic &g);
	};

/* functions prototypes	*/

/* the distance weighting function */
float f(float,int);

#endif
